@@ -2,9 +2,9 @@
 use ahash::AHashMap;
 type HashMap<P, Q> = AHashMap<P, Q>;
 
-use crate::r2_api::{R2Api, Endian, STACK_START, STACK_SIZE};
-use crate::value::Value;
+use crate::r2_api::{Endian, R2Api, STACK_SIZE, STACK_START};
 use crate::solver::Solver;
+use crate::value::Value;
 use std::mem;
 
 // const CHUNK: u64 = 8;
@@ -14,82 +14,81 @@ const READ_CACHE: usize = 64;
 // one day I will make a reasonable heap impl
 // today is not that day
 const HEAP_START: u64 = 0x40000000;
-const HEAP_SIZE:  u64 = 0x04000000;
+const HEAP_SIZE: u64 = 0x04000000;
 // const HEAP_CHUNK: u64 = 0x100;
 
 // pub const CHECK_PERMS: bool = false;
 
 // i think these are different on darwin
-const PROT_NONE:  u64 = 0x0;
-const PROT_READ:  u64 = 0x1;
+const PROT_NONE: u64 = 0x0;
+const PROT_READ: u64 = 0x1;
 const PROT_WRITE: u64 = 0x2;
-const PROT_EXEC:  u64 = 0x4;
+const PROT_EXEC: u64 = 0x4;
 
 #[derive(Clone)]
 pub struct Memory {
     pub solver: Solver,
-    pub r2api:  R2Api,
-    pub mem:    HashMap<u64, Value>,
-    pub heap:   Heap,
-    pub bits:   u64,
+    pub r2api: R2Api,
+    pub mem: HashMap<u64, Value>,
+    pub heap: Heap,
+    pub bits: u64,
     pub endian: Endian,
-    pub segs:   Vec<MemorySegment>,
-    pub blank:  bool,
-    pub check:  bool
+    pub segs: Vec<MemorySegment>,
+    pub blank: bool,
+    pub check: bool,
 }
 
 pub enum Permission {
-    Read, 
-    Write, 
-    Execute
+    Read,
+    Write,
+    Execute,
 }
 
 #[derive(Debug, Clone)]
 pub struct MemorySegment {
-    pub name:  String,
-    pub addr:  u64,
-    pub size:  u64,
-    pub read:  bool,
+    pub name: String,
+    pub addr: u64,
+    pub size: u64,
+    pub read: bool,
     pub write: bool,
-    pub exec:  bool,
-    pub init:  bool
+    pub exec: bool,
+    pub init: bool,
 }
 
 impl Memory {
     pub fn new(r2api: &mut R2Api, btor: Solver, blank: bool, check: bool) -> Memory {
         let segments = r2api.get_segments().unwrap();
-        let mut segs = vec!();
+        let mut segs = vec![];
 
         for seg in segments {
             segs.push(MemorySegment {
-                name:  seg.name,
-                addr:  seg.vaddr,
-                size:  seg.size,
-                read:  seg.perm.contains('r'),
+                name: seg.name,
+                addr: seg.vaddr,
+                size: seg.size,
+                read: seg.perm.contains('r'),
                 write: seg.perm.contains('w'),
-                exec:  seg.perm.contains('x'),
-                init:  true
+                exec: seg.perm.contains('x'),
+                init: true,
             });
         }
 
-        let info = r2api.info.as_ref().unwrap();
-        let endian = info.bin.endian.as_str();
-    
+        let endian = r2api.info.bin.endian.as_str();
+
         Memory {
             solver: btor,
             r2api: r2api.clone(),
             mem: HashMap::new(),
             heap: Heap::new(HEAP_START, HEAP_SIZE),
-            bits: info.bin.bits,
+            bits: r2api.info.bin.bits,
             endian: Endian::from_string(endian),
             segs,
             blank,
-            check
+            check,
         }
     }
 
     pub fn alloc(&mut self, length: &Value) -> u64 {
-        let len = length.as_u64().unwrap(); 
+        let len = length.as_u64().unwrap();
         self.heap.alloc(len)
     }
 
@@ -117,13 +116,13 @@ impl Memory {
     #[inline]
     pub fn check_permission(&self, addr: u64, length: u64, perm: char) -> bool {
         for seg in &self.segs {
-            if addr >= seg.addr && addr + length <= seg.addr+seg.size {
+            if addr >= seg.addr && addr + length <= seg.addr + seg.size {
                 match perm {
                     'r' => return seg.read,
                     'w' => return seg.write,
                     'x' => return seg.exec,
                     'i' => return seg.init,
-                     _  => return false // uhhh shouldnt happen
+                    _ => return false, // uhhh shouldnt happen
                 }
             }
         }
@@ -133,13 +132,13 @@ impl Memory {
 
     pub fn add_segment(&mut self, name: &str, addr: u64, size: u64, perms: &str) {
         self.segs.push(MemorySegment {
-            name:  name.to_owned(),
+            name: name.to_owned(),
             addr,
             size,
-            read:  perms.contains('r'),
+            read: perms.contains('r'),
             write: perms.contains('w'),
-            exec:  perms.contains('x'),
-            init:  perms.contains('i')
+            exec: perms.contains('x'),
+            init: perms.contains('i'),
         });
     }
 
@@ -154,20 +153,26 @@ impl Memory {
     pub fn add_std_streams(&mut self) {
         let mut fd = 0;
         let stds = ["stdin", "stdout", "stderr"];
-        let bits = self.r2api.info.as_ref().unwrap().bin.bits;
         for std in &stds {
             let mut addr = self.r2api.get_address(&("obj.".to_owned() + std)).unwrap();
             let mut offset = 112; // linux
             if addr == 0 {
-                addr = self.r2api.get_address(&("reloc.__".to_owned() + std + "p")).unwrap();
+                addr = self
+                    .r2api
+                    .get_address(&("reloc.__".to_owned() + std + "p"))
+                    .unwrap();
                 offset = 18; // macos, this is jank af
             }
 
             if addr != 0 {
                 // from libc.rs should be in a common place
                 let file_struct = self.alloc(&Value::Concrete(216, 0));
-                self.write_value(file_struct+offset, &Value::Concrete(fd, 0), 4);
-                self.write_value(addr, &Value::Concrete(file_struct, 0), (bits/8) as usize); 
+                self.write_value(file_struct + offset, &Value::Concrete(fd, 0), 4);
+                self.write_value(
+                    addr,
+                    &Value::Concrete(file_struct, 0),
+                    (self.bits / 8) as usize,
+                );
                 fd += 1;
             }
         }
@@ -177,9 +182,9 @@ impl Memory {
         if stk_chk != 0 {
             let chk_value_addr = self.heap.alloc(8);
             self.write_value(
-                chk_value_addr, 
-                &(Value::Symbolic(self.solver.bv("radius_canary", 64), 0)), 
-                (bits/8) as usize
+                chk_value_addr,
+                &(Value::Symbolic(self.solver.bv("radius_canary", 64), 0)),
+                (self.bits / 8) as usize,
             );
         }
     }
@@ -192,22 +197,22 @@ impl Memory {
             if seg.name == ".data" {
                 if avail {
                     // set size and return new break
-                    seg.size = address-seg.addr;
+                    seg.size = address - seg.addr;
                     return address;
                 } else {
                     // return previous break
-                    return seg.addr+seg.size;
+                    return seg.addr + seg.size;
                 }
-            } 
+            }
         }
-        return 0; // if theres no data segment just 0 i guess
+        0
     }
 
     pub fn sbrk(&mut self, inc: u64) -> u64 {
         for seg in &mut self.segs {
             if seg.name == ".data" {
                 seg.size += inc;
-                return seg.addr+seg.size-inc; // returns previous
+                return seg.addr + seg.size - inc; // returns previous
             }
         }
         -1i64 as u64
@@ -216,9 +221,7 @@ impl Memory {
     #[inline]
     pub fn read_sym(&mut self, address: &Value, len: usize, solver: &mut Solver) -> Value {
         match address {
-            Value::Concrete(addr, _t) => {
-                self.read_value(*addr, len)
-            },
+            Value::Concrete(addr, _t) => self.read_value(*addr, len),
             Value::Symbolic(addr, t) => {
                 let addrs = solver.evaluate_many(addr);
                 let mut value = Value::Symbolic(solver.bvv(0, 64), 0);
@@ -237,9 +240,7 @@ impl Memory {
     #[inline]
     pub fn write_sym(&mut self, address: &Value, value: &Value, len: usize, solver: &mut Solver) {
         match address {
-            Value::Concrete(addr, _t) => {
-                self.write_value(*addr, value, len)
-            },
+            Value::Concrete(addr, _t) => self.write_value(*addr, value, len),
             Value::Symbolic(addr, t) => {
                 let addrs = solver.evaluate_many(addr);
                 for a in addrs {
@@ -253,28 +254,37 @@ impl Memory {
         }
     }
 
-    pub fn read_sym_len(&mut self, address: &Value, length: &Value, solver: &mut Solver) -> Vec<Value> {
+    pub fn read_sym_len(
+        &mut self,
+        address: &Value,
+        length: &Value,
+        solver: &mut Solver,
+    ) -> Vec<Value> {
         let len = solver.max_value(length) as usize;
 
         match address {
             Value::Concrete(addr, _t) => {
-                self.read(*addr, len)
-            },
+                let mut data = vec![Value::Concrete(0, 0); len];
+                self.read(*addr, len, &mut data);
+                data
+            }
             Value::Symbolic(addr, t) => {
                 let addrs = solver.evaluate_many(addr);
-                let mut values = vec!();
+                let mut values = vec![];
                 for a in addrs {
-                    let read_vals = self.read(a, len);
-                    let mut new_vals = vec!();
+                    let mut read_vals = vec![Value::Concrete(0, 0); len];
+                    self.read(a, len, &mut read_vals);
+
+                    let mut new_vals = vec![];
                     if values.is_empty() {
                         new_vals = read_vals;
                     } else {
                         for count in 0..len {
                             let bv = solver.bvv(a, 64);
                             let cond = Value::Symbolic(addr._eq(&bv), *t);
-                            let value = solver.conditional(&cond, 
-                                &read_vals[count], &values[count]);
-                            
+                            let value =
+                                solver.conditional(&cond, &read_vals[count], &values[count]);
+
                             new_vals.push(value);
                         }
                     }
@@ -285,38 +295,53 @@ impl Memory {
         }
     }
 
-    pub fn write_sym_len(&mut self, address: &Value, values: &[Value], length: &Value, solver: &mut Solver) {
+    pub fn write_sym_len(
+        &mut self,
+        address: &Value,
+        values: &[Value],
+        length: &Value,
+        solver: &mut Solver,
+    ) {
         let mut len = solver.max_value(length) as usize;
         // hopefully this doesn't occur
         if len > values.len() {
             len = values.len();
         }
 
-        let mut addrs = Vec::with_capacity(128);
+        let mut addrs = Vec::with_capacity(256);
         let t = address.get_taint();
         match address {
             Value::Concrete(addr, _t) => addrs.push(*addr),
-            Value::Symbolic(addr, _t) => addrs.extend(solver.evaluate_many(addr))
+            Value::Symbolic(addr, _t) => addrs.extend(solver.evaluate_many(addr)),
         };
 
         for addr in addrs {
-            let read_vals = self.read(addr, len);
+            let mut read_vals = vec![Value::Concrete(0, 0); len];
+            self.read(addr, len, &mut read_vals);
+
             for count in 0..len {
                 let addr_val = Value::Concrete(addr, t);
-                let count_val = Value::Concrete(count as u64, 0); 
+                let count_val = Value::Concrete(count as u64, 0);
                 let cond = address.eq(&addr_val) & count_val.ult(&length);
                 let value = solver.conditional(&cond, &values[count], &read_vals[count]);
-                self.write(addr+count as u64, &mut [value]);
+                self.write(addr + count as u64, &mut [value]);
             }
         }
     }
 
     pub fn memmove(&mut self, dst: &Value, src: &Value, length: &Value, solver: &mut Solver) {
         let data = self.read_sym_len(src, length, solver);
-        self.write_sym_len(dst, &data, length, solver);    
+        self.write_sym_len(dst, &data, length, solver);
     }
 
-    pub fn search(&mut self, addr: &Value, needle: &Value, length: &Value, reverse: bool, solver: &mut Solver) -> Value {
+    pub fn search(
+        &mut self,
+        addr: &Value,
+        needle: &Value,
+        length: &Value,
+        reverse: bool,
+        solver: &mut Solver,
+    ) -> Value {
         //let mut search_data = self.read_sym_len(addr, length);
         let len = solver.max_value(length) as usize;
 
@@ -333,8 +358,8 @@ impl Memory {
                     mask <<= 8;
                 }
                 l
-            },
-            Value::Symbolic(val, _t) => val.get_width()/8
+            }
+            Value::Symbolic(val, _t) => val.get_width() / 8,
         } as usize;
 
         let mut result = Value::Concrete(0, 0);
@@ -348,8 +373,7 @@ impl Memory {
                 pos_val = addr.clone() + length.clone() - Value::Concrete(pos as u64, 0);
             }
 
-            let pos_cond = pos_val.ult(&(addr.clone() + length.clone())) & 
-                !pos_val.ult(&addr);
+            let pos_cond = pos_val.ult(&(addr.clone() + length.clone())) & !pos_val.ult(&addr);
             let new_cond = value.eq(&needle) & pos_cond & !cond.clone();
             //println!("{:?}", new_cond);
             result = solver.conditional(&new_cond, &pos_val, &result);
@@ -364,20 +388,22 @@ impl Memory {
                 break;
             }
         }
-        
+
         result
     }
 
     pub fn strlen(&mut self, addr: &Value, length: &Value, solver: &mut Solver) -> Value {
         let end = self.search(addr, &Value::Concrete(0, 0), length, false, solver);
-        solver.conditional(
-            &(end.eq(&Value::Concrete(0, 0))), 
-            length,
-            &end.sub(addr)
-        )
+        solver.conditional(&(end.eq(&Value::Concrete(0, 0))), length, &end.sub(addr))
     }
 
-    pub fn compare(&mut self, addr1: &Value, addr2: &Value, length: &Value, solver: &mut Solver) -> Value {
+    pub fn compare(
+        &mut self,
+        addr1: &Value,
+        addr2: &Value,
+        length: &Value,
+        solver: &mut Solver,
+    ) -> Value {
         let len = solver.max_value(length);
         let data1 = self.read_sym_len(addr1, &Value::Concrete(len, length.get_taint()), solver);
         let data2 = self.read_sym_len(addr2, &Value::Concrete(len, length.get_taint()), solver);
@@ -395,12 +421,16 @@ impl Memory {
             let len_cond = ind_val.ult(&length);
 
             let lt_val = solver.conditional(
-                &(lt & same.clone() & len_cond.clone()), 
-                &Value::Concrete(-1i64 as u64, 0), &result);
+                &(lt & same.clone() & len_cond.clone()),
+                &Value::Concrete(-1i64 as u64, 0),
+                &result,
+            );
 
             result = solver.conditional(
-                &(gt & same.clone() & len_cond), 
-                &Value::Concrete(1, 0), &lt_val);
+                &(gt & same.clone() & len_cond),
+                &Value::Concrete(1, 0),
+                &lt_val,
+            );
 
             same = same & result.eq(&Value::Concrete(0, 0));
 
@@ -416,8 +446,15 @@ impl Memory {
 
     #[inline]
     pub fn read_value(&mut self, addr: u64, length: usize) -> Value {
-        let data = self.read(addr, length);
-        self.pack(&data)
+        if length <= 32 {
+            let mut data: [Value; 32] = Default::default();
+            self.read(addr, length, &mut data[..length]);
+            self.pack(&data[..length])
+        } else {
+            let mut data = vec![Value::Concrete(0, 0); length];
+            self.read(addr, length, &mut data);
+            self.pack(&data)
+        }
     }
 
     #[inline]
@@ -426,36 +463,37 @@ impl Memory {
         self.write(addr, &mut data)
     }
 
-    pub fn read(&mut self, addr: u64, length: usize) -> Vec<Value> {
-
+    pub fn read(&mut self, addr: u64, length: usize, data: &mut [Value]) {
+        //println!("length {} data len {}", length, data.());
         if length == 0 {
-            return vec!();
+            return;
         }
 
         if self.check && !self.check_permission(addr, length as u64, 'r') {
-            // everything needs to be reworked to have Result<...> 
+            // everything needs to be reworked to have Result<...>
             // so that we can properly handle things like this
             self.handle_segfault(addr, length as u64, 'r');
-        } 
+        }
 
         let make_sym = self.blank && !self.check_permission(addr, length as u64, 'i');
 
-        let mut data: Vec<Value> = Vec::with_capacity(length);
+        //let mut data: Vec<Value> = Vec::with_capacity(length);
         for count in 0..length as u64 {
             let caddr = addr + count;
             let mem = self.mem.get(&caddr);
 
             match mem {
                 Some(byte) => {
-                    data.push(byte.to_owned());
-                },
+                    data[count as usize] = byte.to_owned();
+                }
                 None => {
                     if make_sym {
                         let sym_name = format!("mem_{:08x}", caddr);
-                        data.push(Value::Symbolic(self.solver.bv(sym_name.as_str(), 8), 0))
+                        data[count as usize] =
+                            Value::Symbolic(self.solver.bv(sym_name.as_str(), 8), 0);
                     } else {
                         let bytes = self.r2api.read(caddr, READ_CACHE).unwrap();
-                        data.push(Value::Concrete(bytes[0] as u64, 0));
+                        data[count as usize] = Value::Concrete(bytes[0] as u64, 0);
                         for (c, byte) in bytes.into_iter().enumerate() {
                             let new_data = Value::Concrete(byte as u64, 0);
                             self.mem.entry(caddr + c as u64).or_insert(new_data);
@@ -465,7 +503,7 @@ impl Memory {
             }
         }
         //println!("read {:?}", data);
-        data
+        //data
     }
 
     pub fn prot_to_str(&self, prot: u64) -> String {
@@ -481,7 +519,7 @@ impl Memory {
             prot_str += "-";
         }
         if prot & PROT_WRITE != 0 {
-            prot_str +=  "w";
+            prot_str += "w";
         } else {
             prot_str += "-";
         }
@@ -490,14 +528,18 @@ impl Memory {
         } else {
             prot_str += "-";
         }
-    
+
         prot_str
     }
 
     pub fn read_bytes(&mut self, addr: u64, length: usize, solver: &mut Solver) -> Vec<u8> {
-        let data = self.read(addr, length);
+        let mut data = vec![Value::Concrete(0, 0); length];
+        self.read(addr, length, &mut data);
         solver.push();
-        let data_u8 = data.iter().map(|d| solver.evalcon_to_u64(&d).unwrap() as u8).collect();
+        let data_u8 = data
+            .iter()
+            .map(|d| solver.evalcon_to_u64(&d).unwrap() as u8)
+            .collect();
         solver.pop();
         data_u8
     }
@@ -509,7 +551,7 @@ impl Memory {
 
     pub fn write_string(&mut self, addr: u64, string: &str) {
         let data = string.as_bytes();
-        let mut data_value = vec!();
+        let mut data_value = vec![];
         for d in data {
             data_value.push(Value::Concrete(*d as u64, 0));
         }
@@ -534,7 +576,10 @@ impl Memory {
     // this sucks, we need to properly do error handling to do this right
     // TODO make everything not suck
     pub fn handle_segfault(&self, addr: u64, length: u64, perm: char) {
-        panic!("addr {} length {} does not have perm \"{}\"", addr, length, perm);
+        panic!(
+            "addr {} length {} does not have perm \"{}\"",
+            addr, length, perm
+        );
     }
 
     // jesus this got huge
@@ -564,9 +609,9 @@ impl Memory {
                         } else {
                             sym_val = val.slice(7, 0).concat(&sym_val);
                         }
-                         
+
                         taint |= t;
-                    },
+                    }
                     Value::Concrete(val, t) => {
                         let new_val = self.solver.bvv(*val, 8);
 
@@ -583,10 +628,9 @@ impl Memory {
             Value::Symbolic(sym_val, taint)
         } else {
             let mut con_val: u64 = 0;
-            for count in 0..length {
-                let datum = new_data.get(count).unwrap();
+            for (count, datum) in new_data.iter().enumerate() {
                 if let Value::Concrete(val, t) = datum {
-                    con_val += val << (8*count);
+                    con_val += val << (8 * count);
                     taint |= t;
                 }
             }
@@ -595,18 +639,18 @@ impl Memory {
     }
 
     pub fn unpack(&self, value: &Value, length: usize) -> Vec<Value> {
-        let mut data: Vec<Value> = Vec::with_capacity(length+1);
+        let mut data: Vec<Value> = Vec::with_capacity(length);
 
         match value {
             Value::Concrete(val, t) => {
                 for count in 0..length {
-                    data.push(Value::Concrete((*val >> (8*count)) & 0xff, *t));
+                    data.push(Value::Concrete((*val >> (8 * count)) & 0xff, *t));
                 }
-            },
+            }
             Value::Symbolic(val, t) => {
                 for count in 0..length {
                     //let trans_val = self.solver.translate(&val).unwrap();
-                    let bv = val.slice(((count as u32)+1)*8-1, (count as u32)*8);
+                    let bv = val.slice(((count as u32) + 1) * 8 - 1, (count as u32) * 8);
                     data.push(Value::Symbolic(bv, *t));
                 }
             }
@@ -626,14 +670,14 @@ impl Memory {
 #[derive(Clone)]
 pub struct Heap {
     pub start: u64,
-    pub size:  u64,
-    pub chunks: Vec<Chunk>
+    pub size: u64,
+    pub chunks: Vec<Chunk>,
 }
 
 #[derive(Clone, PartialEq)]
-pub struct Chunk { 
+pub struct Chunk {
     pub addr: u64,
-    pub size: u64
+    pub size: u64,
 }
 
 // still a dumb af heap implementation
@@ -642,19 +686,22 @@ impl Heap {
         Heap {
             start,
             size,
-            chunks: vec!(Chunk { addr: start, size: 0 })
+            chunks: vec![Chunk {
+                addr: start,
+                size: 0,
+            }],
         }
     }
 
     pub fn alloc(&mut self, size: u64) -> u64 {
-        let last = &self.chunks[self.chunks.len()-1];
-        let addr = last.addr+last.size;
+        let last = &self.chunks[self.chunks.len() - 1];
+        let addr = last.addr + last.size;
         self.chunks.push(Chunk { addr, size });
         addr
     }
 
     pub fn free(&mut self, addr: u64) -> Option<u64> {
-        let last = &self.chunks[self.chunks.len()-1];
+        let last = &self.chunks[self.chunks.len() - 1];
         if addr == last.addr {
             self.chunks.pop();
             Some(addr)
@@ -674,4 +721,4 @@ impl Heap {
             }
         }
     }
-}
\ No newline at end of file
+}
