@@ -1,17 +1,19 @@
-use crate::r2_api::{Instruction, Syscall, CallingConvention, hex_decode};
+use crate::operations::{
+    do_operation, pop_concrete, pop_stack_value, pop_value, push_value, Operations, OPS,
+};
+use crate::r2_api::{hex_decode, CallingConvention, Instruction, Syscall};
 use crate::value::Value;
-use crate::operations::{Operations, pop_value, push_value,
-    pop_stack_value, pop_concrete, do_operation, OPS};
 
-use crate::state::{State, StateStatus, StackItem, 
-    ExecMode, Event, EventTrigger, EventContext, DO_EVENT_HOOKS};
+use crate::state::{
+    Event, EventContext, EventTrigger, ExecMode, StackItem, State, StateStatus, DO_EVENT_HOOKS,
+};
 
-use crate::sims::{SimMethod};
 use crate::sims::syscall::syscall;
+use crate::sims::SimMethod;
 
+use ahash::{AHashMap, AHashSet};
 use std::collections::VecDeque;
 use std::mem;
-use ahash::{AHashMap, AHashSet};
 type HashMap<P, Q> = AHashMap<P, Q>;
 
 const INSTR_NUM: usize = 64;
@@ -25,10 +27,10 @@ pub enum Word {
     Literal(Value),
     Register(usize),
     Operator(Operations),
-    Unknown(String)
+    Unknown(String),
 }
 
-pub type HookMethod = fn (&mut State) -> bool;
+pub type HookMethod = fn(&mut State) -> bool;
 
 #[derive(Clone)]
 pub struct Processor {
@@ -36,7 +38,7 @@ pub struct Processor {
     pub hooks: HashMap<u64, Vec<HookMethod>>,
     pub esil_hooks: HashMap<u64, Vec<String>>,
     pub sims: HashMap<u64, SimMethod>,
-    pub traps: HashMap<u64, SimMethod>, 
+    pub traps: HashMap<u64, SimMethod>,
     pub syscalls: HashMap<u64, Syscall>,
     pub breakpoints: AHashSet<u64>,
     pub mergepoints: AHashSet<u64>,
@@ -49,7 +51,7 @@ pub struct Processor {
     pub lazy: bool,
     pub force: bool,
     pub topological: bool, // execute blocks in topological sort order
-    pub steps: u64         // number of state steps
+    pub steps: u64,        // number of state steps
 }
 
 #[derive(Debug, Clone, PartialEq)]
@@ -60,15 +62,14 @@ pub enum InstructionStatus {
     Sim,
     Merge,
     Avoid,
-    Break
+    Break,
 }
 
 #[derive(Debug, Clone)]
 pub struct InstructionEntry {
     instruction: Instruction,
     tokens: Vec<Word>,
-    status: InstructionStatus
-    // next: Option<Arc<InstructionEntry>>
+    status: InstructionStatus, // next: Option<Arc<InstructionEntry>>
 }
 
 //const DEBUG: bool = false; // show instructions
@@ -79,29 +80,33 @@ pub struct InstructionEntry {
 //const ALLOW_INVALID: bool = true; // Allow invalid instructions (exec as NOP)
 
 impl Processor {
-    pub fn new(selfmodify: bool, optimized: bool, debug: bool, 
-        lazy: bool, force: bool, topological: bool) -> Self {
-
+    pub fn new(
+        selfmodify: bool,
+        optimized: bool,
+        debug: bool,
+        lazy: bool,
+        force: bool,
+        topological: bool,
+    ) -> Self {
         Processor {
             instructions: HashMap::new(),
-            hooks:        HashMap::new(),
-            esil_hooks:   HashMap::new(),
-            sims:         HashMap::new(),
-            traps:        HashMap::new(),
-            syscalls:     HashMap::new(),
-            breakpoints:  AHashSet::new(),
-            mergepoints:  AHashSet::new(),
-            avoidpoints:  AHashSet::new(),
+            hooks: HashMap::new(),
+            esil_hooks: HashMap::new(),
+            sims: HashMap::new(),
+            traps: HashMap::new(),
+            syscalls: HashMap::new(),
+            breakpoints: AHashSet::new(),
+            mergepoints: AHashSet::new(),
+            avoidpoints: AHashSet::new(),
             //visited:      AHashSet::new(),
-            merges:       HashMap::new(),
+            merges: HashMap::new(),
             selfmodify,
             optimized,
             debug,
             lazy,
             force,
             topological,
-            steps: 0
-            //states: vec!()
+            steps: 0, //states: vec!()
         }
     }
 
@@ -121,32 +126,28 @@ impl Processor {
                 tokens.push(operator);
 
             // all this garbage is for the combo ones like ++=[8] ...
-            } else if l > 1 && &s[l-1..] == "="
-                    && OPS.contains(&&s[0..l-1]) {
-
+            } else if l > 1 && &s[l - 1..] == "=" && OPS.contains(&&s[0..l - 1]) {
                 let reg_word = tokens.pop().unwrap();
                 tokens.push(reg_word.to_owned());
-                let operator = self.get_operator(&s[0..l-1]).unwrap();
+                let operator = self.get_operator(&s[0..l - 1]).unwrap();
                 tokens.push(operator);
                 tokens.push(reg_word);
                 tokens.push(Word::Operator(Operations::Equal));
-
-            } else if l > 4 && &s[l-1..] == "]" 
-                    && OPS.contains(&&s[0..l-4]) {
-
+            } else if l > 4 && &s[l - 1..] == "]" && OPS.contains(&&s[0..l - 4]) {
                 tokens.push(Word::Operator(Operations::AddressStore));
-                let peek = self.get_operator(&s[l-3..]).unwrap();
+                let peek = self.get_operator(&s[l - 3..]).unwrap();
                 tokens.push(peek);
-                let operator = self.get_operator(&s[0..l-4]).unwrap();
+                let operator = self.get_operator(&s[0..l - 4]).unwrap();
                 tokens.push(operator);
-                let poke = self.get_operator(&s[l-4..]).unwrap();
+                let poke = self.get_operator(&s[l - 4..]).unwrap();
                 tokens.push(Word::Operator(Operations::AddressRestore));
                 tokens.push(poke);
-            
             } else if let Some(values) = state.context.get(s) {
-                tokens.extend(values.iter()
-                    .map(|x| Word::Literal(state.translate_value(x))));
-
+                tokens.extend(
+                    values
+                        .iter()
+                        .map(|x| Word::Literal(state.translate_value(x))),
+                );
             } else {
                 tokens.push(Word::Unknown(String::from(s)));
             }
@@ -156,7 +157,7 @@ impl Processor {
     }
 
     /// attempt to tokenize word as number literal (eg. 0x8)
-    pub fn get_literal(&self, word: &str) -> Option<Word> {        
+    pub fn get_literal(&self, word: &str) -> Option<Word> {
         if let Ok(i) = word.parse::<u64>() {
             Some(Word::Literal(Value::Concrete(i, 0)))
         } else if word.len() > 2 && &word[0..2] == "0x" {
@@ -170,29 +171,41 @@ impl Processor {
     }
 
     /// attempt to tokenize word as register (eg. rbx)
-    pub fn get_register(&self,  state: &mut State, word: &str) -> Option<Word> {
+    pub fn get_register(&self, state: &mut State, word: &str) -> Option<Word> {
         let name = if let Some(alias) = state.registers.aliases.get(word) {
             alias.reg.as_str()
         } else {
             word
         };
-        state.registers.get_register(name).map(|reg| Word::Register(reg.index))
+        state
+            .registers
+            .get_register(name)
+            .map(|reg| Word::Register(reg.index))
     }
 
     /// attempt to tokenize word as operation (eg. +)
     pub fn get_operator(&self, word: &str) -> Option<Word> {
         match Operations::from_string(word) {
             Operations::Unknown => None,
-            op => Some(Word::Operator(op))
+            op => Some(Word::Operator(op)),
         }
     }
 
     /// print instruction if debug output is enabled
     pub fn print_instr(&self, state: &mut State, instr: &Instruction) {
         if !COLOR {
-            println!("{:016x}:  {:<40} |  {}", instr.offset, instr.disasm, instr.esil);
+            println!(
+                "{:016x}:  {:<40} |  {}",
+                instr.offset, instr.disasm, instr.esil
+            );
         } else {
-            print!("{}", state.r2api.cmd(&format!("pd 1 @ {}", instr.offset)).unwrap());
+            print!(
+                "{}",
+                state
+                    .r2api
+                    .cmd(&format!("pd 1 @ {}", instr.offset))
+                    .unwrap()
+            );
         }
     }
 
@@ -220,16 +233,16 @@ impl Processor {
     }
 
     /**
-     * Parse and execute the vector of tokenized ESIL words. 
+     * Parse and execute the vector of tokenized ESIL words.
      * The difficult parts here are the temporary stacks for IF/ELSE
      * When a conditional is symbolic the stack needs to be copied
      * into separate stacks for the if and else portions
-     * after ENDIF (}) these stacks are unwound into a single vec of 
+     * after ENDIF (}) these stacks are unwound into a single vec of
      * conditional bitvectors IF(cond, IF_VAL, ELSE_VAL)
      */
     pub fn parse(&self, state: &mut State, words: &[Word]) {
         state.stack.clear();
-        
+
         let mut word_index = 0;
         let words_len = words.len();
 
@@ -237,27 +250,26 @@ impl Processor {
             let word = &words[word_index];
             word_index += 1;
 
-            // this is weird... 
+            // this is weird...
             if state.esil.mode == ExecMode::NoExec {
                 match &word {
-                    Word::Operator(Operations::Else) | 
-                    Word::Operator(Operations::EndIf) => {},
-                    _ => continue
+                    Word::Operator(Operations::Else) | Word::Operator(Operations::EndIf) => {}
+                    _ => continue,
                 }
             }
 
             match word {
                 Word::Literal(val) => {
                     state.stack.push(StackItem::StackValue(val.to_owned()));
-                },
+                }
                 Word::Register(index) => {
                     state.stack.push(StackItem::StackRegister(*index));
-                },
+                }
                 Word::Operator(op) => {
                     match op {
                         Operations::If => {
                             let arg1 = pop_value(state, false, false);
-                
+
                             match (arg1, &state.esil.mode) {
                                 (Value::Concrete(val1, _t), ExecMode::Uncon) => {
                                     state.esil.mode = if val1 == 0 {
@@ -265,13 +277,12 @@ impl Processor {
                                     } else {
                                         ExecMode::Exec
                                     };
-                                },
+                                }
                                 (Value::Symbolic(val1, _t), ExecMode::Uncon) => {
                                     //println!("if {:?}", val1);
                                     state.esil.mode = ExecMode::If;
                                     state.esil.temp1 = state.stack.to_owned();
-                                    let cond_bv = val1._eq(
-                                        &state.bvv(0, val1.get_width())).not();
+                                    let cond_bv = val1._eq(&state.bvv(0, val1.get_width())).not();
 
                                     state.condition = Some(cond_bv);
                                 }
@@ -279,38 +290,31 @@ impl Processor {
                                     println!("Bad ESIL?");
                                 }
                             }
-                        },
-                        Operations::Else => {
-                            match &state.esil.mode {
-                                ExecMode::Exec => state.esil.mode = ExecMode::NoExec,
-                                ExecMode::NoExec => state.esil.mode = ExecMode::Exec,
-                                ExecMode::If => {
-                                    state.esil.mode = ExecMode::Else;
-                                    state.condition = Some(state.condition.as_ref().unwrap().not());
-                                    state.esil.temp2 = mem::take(&mut state.stack); 
-                                    state.stack = mem::take(&mut state.esil.temp1); 
-                                }
-                                _ => {}
+                        }
+                        Operations::Else => match &state.esil.mode {
+                            ExecMode::Exec => state.esil.mode = ExecMode::NoExec,
+                            ExecMode::NoExec => state.esil.mode = ExecMode::Exec,
+                            ExecMode::If => {
+                                state.esil.mode = ExecMode::Else;
+                                state.condition = Some(state.condition.as_ref().unwrap().not());
+                                state.esil.temp2 = mem::take(&mut state.stack);
+                                state.stack = mem::take(&mut state.esil.temp1);
                             }
+                            _ => {}
                         },
                         Operations::EndIf => {
-
                             match &state.esil.mode {
-                                ExecMode::If | ExecMode::Else => {},
+                                ExecMode::If | ExecMode::Else => {}
                                 _ => {
                                     state.esil.mode = ExecMode::Uncon;
                                     continue;
                                 }
                             };
-                            
+
                             let mut new_temp = match &state.esil.mode {
-                                ExecMode::If => {
-                                    mem::take(&mut state.esil.temp1)
-                                },
-                                ExecMode::Else => {
-                                    mem::take(&mut state.esil.temp2)
-                                },
-                                _ => vec!() // won't happen
+                                ExecMode::If => mem::take(&mut state.esil.temp1),
+                                ExecMode::Else => mem::take(&mut state.esil.temp2),
+                                _ => vec![], // won't happen
                             };
 
                             // this is weird but just a trick to not have to alloc a new vec
@@ -320,9 +324,12 @@ impl Processor {
                                 let if_val = pop_stack_value(state, &mut old_stack, false, false);
                                 let else_val = pop_stack_value(state, &mut new_temp, false, false);
                                 let cond_val = state.solver.conditional(
-                                    &Value::Symbolic(state.condition.as_ref().unwrap().to_owned(), 0),
+                                    &Value::Symbolic(
+                                        state.condition.as_ref().unwrap().to_owned(),
+                                        0,
+                                    ),
                                     &if_val,
-                                    &else_val
+                                    &else_val,
                                 );
 
                                 new_stack.push(StackItem::StackValue(cond_val));
@@ -331,9 +338,9 @@ impl Processor {
                             new_stack.reverse();
                             state.stack = new_stack;
                             state.condition = None;
-                        
+
                             state.esil.mode = ExecMode::Uncon;
-                        },
+                        }
                         Operations::GoTo => {
                             let n = pop_concrete(state, false, false);
                             if let Some(_cond) = &state.condition {
@@ -344,7 +351,7 @@ impl Processor {
                             }
                             state.esil.mode = ExecMode::Uncon;
                             word_index = n as usize;
-                        },
+                        }
                         Operations::Break => {
                             if let Some(_cond) = &state.condition {
                                 println!("Hit symbolic BREAK");
@@ -353,27 +360,27 @@ impl Processor {
                             }
                             state.esil.mode = ExecMode::Uncon;
                             break;
-                        },
+                        }
                         Operations::Trap => {
                             let trap = pop_concrete(state, false, false);
                             //let pc = state.registers.get_pc().as_u64().unwrap();
 
-                            let sys_val = state.registers.get_with_alias("SN");                            
+                            let sys_val = state.registers.get_with_alias("SN");
                             if let Some(trap_sim) = self.traps.get(&trap) {
                                 // provide syscall args
                                 let cc = state.r2api.get_syscall_cc().unwrap();
-                                let mut args = vec!(sys_val);
+                                let mut args = vec![sys_val];
                                 for arg in cc.args {
                                     args.push(state.registers.get(arg.as_str()));
                                 }
                                 let ret = trap_sim(state, &args);
                                 state.registers.set(cc.ret.as_str(), ret);
                             }
-                        },
+                        }
                         Operations::Syscall => self.do_syscall(state),
-                        _ => do_operation(state, op)
+                        _ => do_operation(state, op),
                     }
-                },
+                }
                 Word::Unknown(s) => {
                     push_value(state, Value::Concrete(0, 0));
                     println!("Unknown word: {}", s);
@@ -385,8 +392,12 @@ impl Processor {
     /// removes words that weak set flag values that are never read, and words that are NOPs
     pub fn optimize(&mut self, state: &mut State, prev_pc: u64, curr_instr: &InstructionEntry) {
         let prev_instr = &self.instructions[&prev_pc];
-        if  !prev_instr.tokens.contains(&Word::Operator(Operations::WeakEqual)) ||
-            !curr_instr.tokens.contains(&Word::Operator(Operations::WeakEqual))
+        if !prev_instr
+            .tokens
+            .contains(&Word::Operator(Operations::WeakEqual))
+            || !curr_instr
+                .tokens
+                .contains(&Word::Operator(Operations::WeakEqual))
         {
             return;
         }
@@ -397,8 +408,8 @@ impl Processor {
         let len = curr_instr.tokens.len();
         for (i, word) in curr_instr.tokens.iter().enumerate() {
             if let Word::Register(index) = word {
-                if i+1 < len {
-                    let next = &curr_instr.tokens[i+1];
+                if i + 1 < len {
+                    let next = &curr_instr.tokens[i + 1];
                     if let Word::Operator(op) = next {
                         if let Operations::WeakEqual = op {
                             regs_written.push(*index);
@@ -420,20 +431,28 @@ impl Processor {
                 if let Operations::NoOperation = op {
                     remove.push(i); // remove nops
                 } else if let Operations::WeakEqual = op {
-                    let reg = &prev_instr.tokens[i-1];
+                    let reg = &prev_instr.tokens[i - 1];
                     if let Word::Register(index) = reg {
-                        if !regs_read.iter().any(|r| state.registers.is_sub(*r, *index)) &&
-                            regs_written.iter().any(|r| state.registers.is_sub(*r, *index)) {
-
-                            let val = &prev_instr.tokens[i-2];
+                        if !regs_read.iter().any(|r| state.registers.is_sub(*r, *index))
+                            && regs_written
+                                .iter()
+                                .any(|r| state.registers.is_sub(*r, *index))
+                        {
+                            let val = &prev_instr.tokens[i - 2];
                             if let Word::Operator(op) = val {
                                 match op {
-                                    Operations::Zero => remove.extend(vec!(i-2, i-1, i)),
-                                    Operations::Carry => remove.extend(vec!(i-3, i-2, i-1, i)),
-                                    Operations::Borrow => remove.extend(vec!(i-3, i-2, i-1, i)),
-                                    Operations::Parity => remove.extend(vec!(i-2, i-1, i)),
-                                    Operations::Overflow => remove.extend(vec!(i-3, i-2, i-1, i)),
-                                    Operations::S => remove.extend(vec!(i-3, i-2, i-1, i)),
+                                    Operations::Zero => remove.extend(vec![i - 2, i - 1, i]),
+                                    Operations::Carry => {
+                                        remove.extend(vec![i - 3, i - 2, i - 1, i])
+                                    }
+                                    Operations::Borrow => {
+                                        remove.extend(vec![i - 3, i - 2, i - 1, i])
+                                    }
+                                    Operations::Parity => remove.extend(vec![i - 2, i - 1, i]),
+                                    Operations::Overflow => {
+                                        remove.extend(vec![i - 3, i - 2, i - 1, i])
+                                    }
+                                    Operations::S => remove.extend(vec![i - 3, i - 2, i - 1, i]),
                                     _ => {}
                                 }
                             }
@@ -458,17 +477,19 @@ impl Processor {
     }
 
     pub fn get_args(&self, state: &mut State, cc: &CallingConvention) -> Vec<Value> {
-        let mut args = Vec::with_capacity(16); 
+        let mut args = Vec::with_capacity(16);
 
-        if cc.args.len() > 0 {
+        if !cc.args.is_empty() {
             for arg in &cc.args {
                 args.push(state.registers.get_with_alias(arg));
             }
-        } else { // read args from stack?
+        } else {
+            // read args from stack?
             let mut sp = state.registers.get_with_alias("SP");
-            let length = state.memory.bits as usize/8; 
+            let length = state.memory.bits as usize / 8;
 
-            for _ in 0..8 { // do 8 idk?
+            for _ in 0..8 {
+                // do 8 idk?
                 sp = sp + Value::Concrete(length as u64, 0);
                 let value = state.memory_read_value(&sp, length);
                 args.push(value);
@@ -483,9 +504,13 @@ impl Processor {
      * If the instruction is hooked or the method is simulated perform the
      * respective callback. Hooks returning false will skip the instruction
      */
-    pub fn execute(&self, state: &mut State, instr: &Instruction, 
-        status: &InstructionStatus, words: &[Word]) {
-
+    pub fn execute(
+        &self,
+        state: &mut State,
+        instr: &Instruction,
+        status: &InstructionStatus,
+        words: &[Word],
+    ) {
         if state.memory.check && !state.memory.check_permission(instr.offset, instr.size, 'x') {
             state.memory.handle_segfault(instr.offset, instr.size, 'x');
         }
@@ -500,32 +525,30 @@ impl Processor {
             if instr.fail != 0 {
                 state.esil.pcs.push(instr.fail as u64);
             }
-        } 
+        }
 
         // shit is gettin messy
         let mut new_status = status;
-        if state.status == StateStatus::PostMerge && 
-            *status == InstructionStatus::Merge {
-
+        if state.status == StateStatus::PostMerge && *status == InstructionStatus::Merge {
             state.status = StateStatus::Active;
             new_status = &InstructionStatus::None;
         }
 
         match instr.type_num {
             CALL_TYPE => state.backtrace.push((instr.jump as u64, new_pc)),
-            RETN_TYPE => { 
-                if state.backtrace.is_empty() && 
-                    new_status == &InstructionStatus::None {
+            RETN_TYPE => {
+                if state.backtrace.is_empty() && new_status == &InstructionStatus::None {
                     // try to avoid returning outside valid context
                     if !self.breakpoints.is_empty() {
                         new_status = &InstructionStatus::Avoid;
-                    } else { // break if there are no other breakpoints
+                    } else {
+                        // break if there are no other breakpoints
                         new_status = &InstructionStatus::Break;
                     }
                 } else {
                     state.backtrace.pop();
                 }
-            },
+            }
             _ => {}
         }
 
@@ -534,7 +557,7 @@ impl Processor {
                 let pc_val = Value::Concrete(new_pc, 0);
                 state.registers.set_pc(pc_val);
                 self.parse(state, words);
-            },
+            }
             InstructionStatus::Hook => {
                 let mut skip = false;
                 let pc_val = Value::Concrete(new_pc, 0);
@@ -548,7 +571,7 @@ impl Processor {
                 if !skip {
                     self.parse(state, words);
                 }
-            },
+            }
             InstructionStatus::ESILHook => {
                 let mut skip = false;
                 let pc_val = Value::Concrete(new_pc, 0);
@@ -564,7 +587,7 @@ impl Processor {
                 if !skip {
                     self.parse(state, words);
                 }
-            },
+            }
             InstructionStatus::Sim => {
                 let sim = &self.sims[&pc];
                 let cc = state.r2api.get_cc(pc).unwrap_or_default();
@@ -585,10 +608,10 @@ impl Processor {
                         self.ret(state);
                     }
                 }
-            },
+            }
             InstructionStatus::Break => state.status = StateStatus::Break,
             InstructionStatus::Merge => state.status = StateStatus::Merge,
-            InstructionStatus::Avoid => state.status = StateStatus::Inactive
+            InstructionStatus::Avoid => state.status = StateStatus::Inactive,
         };
     }
 
@@ -603,21 +626,20 @@ impl Processor {
     pub fn fetch_instruction(&mut self, state: &mut State, pc_val: u64) {
         let has_instr = self.instructions.contains_key(&pc_val);
         if self.selfmodify || !has_instr {
-
             let mut pc_tmp = pc_val;
             let instrs = if self.selfmodify {
-                let data =  state.memory_read_bytes(pc_val, 32);
+                let data = state.memory_read_bytes(pc_val, 32);
                 // 1 at a time for selfmodify
                 // check to see if bytes changed
                 if has_instr {
                     let instr = &self.instructions[&pc_val];
                     let bytes = hex_decode(&instr.instruction.bytes);
                     if bytes == data[..bytes.len()].to_vec() {
-                        // nothing needs to be done but this is 
+                        // nothing needs to be done but this is
                         // such a weird construction. i hate this code
-                        return; 
+                        return;
                     }
-                } 
+                }
                 state.r2api.disassemble_bytes(pc_val, &data, 1).unwrap()
             } else {
                 state.r2api.disassemble(pc_val, INSTR_NUM).unwrap()
@@ -652,7 +674,7 @@ impl Processor {
                 let instr_entry = InstructionEntry {
                     instruction: instr,
                     tokens: words,
-                    status
+                    status,
                 };
 
                 if opt {
@@ -672,7 +694,7 @@ impl Processor {
 
         // the hash lookup is done twice, needs fixing
         let instr = self.instructions.get(&pc_val).unwrap();
-        
+
         if self.debug {
             self.print_instr(state, &instr.instruction);
         }
@@ -711,13 +733,19 @@ impl Processor {
             }
 
             if DO_EVENT_HOOKS && state.has_event_hooks {
-                state.do_hooked(&Event::SymbolicExec(EventTrigger::Before), 
-                    &EventContext::ExecContext(new_pc.clone()));
+                state.do_hooked(
+                    &Event::SymbolicExec(EventTrigger::Before),
+                    &EventContext::ExecContext(new_pc.clone()),
+                );
             }
-            
+
             if !self.lazy && !state.esil.pcs.is_empty() {
                 // testing sat without modelgen is a bit faster than evaluating
-                state.esil.pcs = state.esil.pcs.clone().into_iter()
+                state.esil.pcs = state
+                    .esil
+                    .pcs
+                    .clone()
+                    .into_iter()
                     .filter(|x| state.check(&new_pc.eq(&Value::Concrete(*x, 0))))
                     .collect();
             } else if state.esil.pcs.is_empty() {
@@ -725,15 +753,17 @@ impl Processor {
             }
 
             if DO_EVENT_HOOKS && state.has_event_hooks {
-                state.do_hooked(&Event::SymbolicExec(EventTrigger::After), 
-                    &EventContext::ExecContext(new_pc.clone()));
+                state.do_hooked(
+                    &Event::SymbolicExec(EventTrigger::After),
+                    &EventContext::ExecContext(new_pc.clone()),
+                );
             }
         }
 
         if state.esil.pcs.len() > 1 || new_pc.as_u64().is_none() {
             let mut states: Vec<State> = Vec::with_capacity(pc_allocs);
 
-            let last = state.esil.pcs.len()-1;
+            let last = state.esil.pcs.len() - 1;
             for new_pc_val in &state.esil.pcs[..last] {
                 let mut new_state = state.clone();
                 if let Some(pc_val) = new_pc.as_bv() {
@@ -743,17 +773,17 @@ impl Processor {
                 new_state.registers.set_pc(Value::Concrete(*new_pc_val, 0));
                 states.push(new_state);
             }
-            
+
             let new_pc_val = state.esil.pcs[last];
             if let Some(pc_val) = new_pc.as_bv() {
-                let pc_bv = pc_val; 
+                let pc_bv = pc_val;
                 let a = pc_bv._eq(&state.bvv(new_pc_val, pc_bv.get_width()));
                 state.solver.assert(&a);
             }
             state.registers.set_pc(Value::Concrete(new_pc_val, 0));
             states
         } else {
-            vec!()
+            vec![]
         }
     }
 
@@ -769,14 +799,14 @@ impl Processor {
                 if self.merges.is_empty() {
                     return VecDeque::new();
                 } else {
-                    // pop one out of mergers 
+                    // pop one out of mergers
                     let key = *self.merges.keys().next().unwrap();
                     let mut merge = self.merges.remove(&key).unwrap();
                     merge.status = StateStatus::PostMerge;
                     states.push_back(merge)
                 }
-            } 
-            
+            }
+
             let current_state = &mut states[index];
 
             match current_state.status {
@@ -787,22 +817,26 @@ impl Processor {
                         index += 1;
                     }
                     index += 1;
-                },
+                }
                 StateStatus::Merge => {
                     self.merge(states.remove(index).unwrap());
-                },
+                }
                 StateStatus::Break => {
                     if current_state.is_sat() {
-                        return VecDeque::from(vec!(states.remove(index).unwrap())); 
+                        return VecDeque::from(vec![states.remove(index).unwrap()]);
                     } else {
                         states.remove(index).unwrap();
                     }
-                },
+                }
                 _ => {
                     states.remove(index).unwrap();
                 }
             }
-            index = if states.len() == 0 { 0 } else { index%states.len() };
+            index = if states.is_empty() {
+                0
+            } else {
+                index % states.len()
+            };
         }
 
         states
@@ -810,9 +844,10 @@ impl Processor {
 
     pub fn merge(&mut self, mut state: State) {
         let pc = state.registers.get_with_alias("PC").as_u64().unwrap();
-        
-        let has_pc = self.merges.contains_key(&pc); 
-        if !has_pc { // trick clippy idk
+
+        let has_pc = self.merges.contains_key(&pc);
+        if !has_pc {
+            // trick clippy idk
             self.merges.insert(pc, state);
         } else {
             let mut merge_state = self.merges.remove(&pc).unwrap();
@@ -820,17 +855,17 @@ impl Processor {
             let assertion = state.solver.and_all(&state_asserts);
             let asserted = Value::Symbolic(assertion.clone(), 0);
 
-            // merge registers 
+            // merge registers
             let mut new_regs = Vec::with_capacity(256);
             let reg_count = state.registers.values.len();
             for index in 0..reg_count {
                 let reg = &merge_state.registers.values[index];
-                let curr_reg  = state.registers.values[index].clone();
+                let curr_reg = state.registers.values[index].clone();
                 new_regs.push(state.solver.conditional(&asserted, &curr_reg, &reg));
             }
             merge_state.registers.values = new_regs;
 
-            // merge memory 
+            // merge memory
             let mut new_mem = HashMap::new();
             let merge_addrs = merge_state.memory.addresses();
             let state_addrs = state.memory.addresses();
