@@ -1,11 +1,10 @@
-
-use boolector::{Btor, BV};
 use crate::solver::BitVec;
-use std::sync::Arc;
-use std::ops;
+use boolector::{Btor, BV};
 use std::cmp::Ordering;
+use std::ops;
+use std::sync::Arc;
 
-#[inline] 
+#[inline]
 pub fn log2(x: u32) -> u32 {
     31 - x.leading_zeros()
 }
@@ -13,33 +12,33 @@ pub fn log2(x: u32) -> u32 {
 #[derive(Debug, Clone, PartialEq)]
 pub enum Value {
     Concrete(u64, u64),
-    Symbolic(BitVec, u64)
+    Symbolic(BitVec, u64),
 }
 
 impl Default for Value {
-    fn default() -> Self { Value::Concrete(0, 0) }
+    fn default() -> Self {
+        Value::Concrete(0, 0)
+    }
 }
 
-#[inline] 
+#[inline]
 pub fn make_bv(bv: &BitVec, val: u64, n: u32) -> BitVec {
     BV::from_u64(bv.get_btor(), val, n)
 }
 
-#[inline] 
+#[inline]
 pub fn value_to_bv(btor: Arc<Btor>, value: Value) -> BitVec {
     match value {
-        Value::Concrete(val, _t) => {
-            BV::from_u64(btor, val, 64)
-        },
-        Value::Symbolic(val, _t) => val 
+        Value::Concrete(val, _t) => BV::from_u64(btor, val, 64),
+        Value::Symbolic(val, _t) => val,
     }
 }
 
-#[inline] 
+#[inline]
 pub fn cond_value(cond: &BitVec, v1: Value, v2: Value) -> BitVec {
     cond.cond_bv(
-        &value_to_bv(cond.get_btor(), v1), 
-        &value_to_bv(cond.get_btor(), v2)
+        &value_to_bv(cond.get_btor(), v1),
+        &value_to_bv(cond.get_btor(), v2),
     )
 }
 
@@ -74,21 +73,25 @@ macro_rules! wrapping_binary_ops {
         match ($self, $rhs) {
             (Value::Concrete(a, t1), Value::Concrete(b, t2)) => {
                 Value::Concrete(a.$wrapping(*b), *t1 | *t2)
-            },
+            }
             (Value::Symbolic(a, t1), Value::Concrete(b, t2)) => {
                 let bv = make_bv(a, *b, a.get_width());
                 Value::Symbolic(a.$method(&bv), *t1 | *t2)
-            },
+            }
             (Value::Concrete(a, t1), Value::Symbolic(b, t2)) => {
                 let bv = make_bv(b, *a, b.get_width());
                 Value::Symbolic(bv.$method(&b), *t1 | *t2)
-            },
+            }
             (Value::Symbolic(a, t1), Value::Symbolic(b, t2)) => {
                 let width_diff = a.get_width() as i32 - b.get_width() as i32;
                 match width_diff.cmp(&0) {
                     Ordering::Equal => Value::Symbolic(a.$method(&b), *t1 | *t2),
-                    Ordering::Greater => Value::Symbolic(a.$method(&b.uext(width_diff as u32)), *t1 | *t2),
-                    Ordering::Less => Value::Symbolic(a.uext((-width_diff) as u32).$method(&b), *t1 | *t2)
+                    Ordering::Greater => {
+                        Value::Symbolic(a.$method(&b.uext(width_diff as u32)), *t1 | *t2)
+                    }
+                    Ordering::Less => {
+                        Value::Symbolic(a.uext((-width_diff) as u32).$method(&b), *t1 | *t2)
+                    }
                 }
             }
         }
@@ -98,7 +101,7 @@ macro_rules! wrapping_binary_ops {
 impl ops::Add<Value> for Value {
     type Output = Value;
 
-    #[inline] 
+    #[inline]
     fn add(self, rhs: Value) -> Value {
         wrapping_binary_ops!(&self, &rhs, add, wrapping_add)
     }
@@ -107,7 +110,7 @@ impl ops::Add<Value> for Value {
 impl ops::Sub<Value> for Value {
     type Output = Value;
 
-    #[inline] 
+    #[inline]
     fn sub(self, rhs: Value) -> Value {
         wrapping_binary_ops!(&self, &rhs, sub, wrapping_sub)
     }
@@ -116,7 +119,7 @@ impl ops::Sub<Value> for Value {
 impl ops::Mul<Value> for Value {
     type Output = Value;
 
-    #[inline] 
+    #[inline]
     fn mul(self, rhs: Value) -> Value {
         wrapping_binary_ops!(&self, &rhs, mul, wrapping_mul)
     }
@@ -125,7 +128,7 @@ impl ops::Mul<Value> for Value {
 impl ops::Div<Value> for Value {
     type Output = Value;
 
-    #[inline] 
+    #[inline]
     fn div(self, rhs: Value) -> Value {
         wrapping_binary_ops!(&self, &rhs, udiv, wrapping_div)
     }
@@ -134,7 +137,7 @@ impl ops::Div<Value> for Value {
 impl ops::Rem<Value> for Value {
     type Output = Value;
 
-    #[inline] 
+    #[inline]
     fn rem(self, rhs: Value) -> Value {
         wrapping_binary_ops!(&self, &rhs, urem, wrapping_rem)
     }
@@ -143,7 +146,7 @@ impl ops::Rem<Value> for Value {
 impl ops::BitAnd<Value> for Value {
     type Output = Value;
 
-    #[inline] 
+    #[inline]
     fn bitand(self, rhs: Value) -> Value {
         binary_ops!(&self, &rhs, and, &)
     }
@@ -152,7 +155,7 @@ impl ops::BitAnd<Value> for Value {
 impl ops::BitOr<Value> for Value {
     type Output = Value;
 
-    #[inline] 
+    #[inline]
     fn bitor(self, rhs: Value) -> Value {
         binary_ops!(&self, &rhs, or, |)
     }
@@ -164,26 +167,28 @@ impl ops::BitXor<Value> for Value {
     // ok here we could clear taint if a == b since this is used to zero regs
     // idk if this is actually a good idea or not. but something needs to be done
     // to stop wild overtainting
-    #[inline] 
+    #[inline]
     fn bitxor(self, rhs: Value) -> Value {
         match (self, rhs) {
             (Value::Concrete(a, t1), Value::Concrete(b, t2)) => {
                 Value::Concrete(a ^ b, (t1 | t2) * ((a != b || t1 != t2) as u64))
-            },
+            }
             (Value::Symbolic(a, t1), Value::Concrete(b, t2)) => {
                 let bv = make_bv(&a, b, a.get_width());
                 Value::Symbolic(a.xor(&bv), t1 | t2)
-            },
+            }
             (Value::Concrete(a, t1), Value::Symbolic(b, t2)) => {
                 let bv = make_bv(&b, a, b.get_width());
                 Value::Symbolic(bv.xor(&b), t1 | t2)
-            },
+            }
             (Value::Symbolic(a, t1), Value::Symbolic(b, t2)) => {
                 let width_diff = a.get_width() as i32 - b.get_width() as i32;
                 match width_diff.cmp(&0) {
                     Ordering::Equal => Value::Symbolic(a.xor(&b), t1 | t2),
-                    Ordering::Greater => Value::Symbolic(a.xor(&b.uext(width_diff as u32)), t1 | t2),
-                    Ordering::Less => Value::Symbolic(a.uext(-width_diff as u32).xor(&b), t1 | t2)
+                    Ordering::Greater => {
+                        Value::Symbolic(a.xor(&b.uext(width_diff as u32)), t1 | t2)
+                    }
+                    Ordering::Less => Value::Symbolic(a.uext(-width_diff as u32).xor(&b), t1 | t2),
                 }
             }
         }
@@ -193,15 +198,13 @@ impl ops::BitXor<Value> for Value {
 impl ops::Not for Value {
     type Output = Value;
 
-    #[inline] 
+    #[inline]
     fn not(self) -> Value {
         match self {
-            Value::Concrete(a, t) => {
-                Value::Concrete((a == 0) as u64, t)
-            },
+            Value::Concrete(a, t) => Value::Concrete((a == 0) as u64, t),
             Value::Symbolic(a, t) => {
                 let zero = BV::zero(a.get_btor(), a.get_width());
-                Value::Symbolic(a._eq(&zero).uext(a.get_width()-1), t)
+                Value::Symbolic(a._eq(&zero).uext(a.get_width() - 1), t)
             }
         }
     }
@@ -210,15 +213,11 @@ impl ops::Not for Value {
 impl ops::Neg for Value {
     type Output = Value;
 
-    #[inline] 
+    #[inline]
     fn neg(self) -> Value {
         match self {
-            Value::Concrete(a, t) => {
-                Value::Concrete(a.wrapping_neg(), t)
-            },
-            Value::Symbolic(a, t) => {
-                Value::Symbolic(a.neg(), t)
-            }
+            Value::Concrete(a, t) => Value::Concrete(a.wrapping_neg(), t),
+            Value::Symbolic(a, t) => Value::Symbolic(a.neg(), t),
         }
     }
 }
@@ -226,22 +225,22 @@ impl ops::Neg for Value {
 impl ops::Shl<Value> for Value {
     type Output = Value;
 
-    #[inline] 
+    #[inline]
     fn shl(self, rhs: Value) -> Value {
         match (self, rhs) {
             (Value::Concrete(a, t1), Value::Concrete(b, t2)) => {
                 Value::Concrete(a.wrapping_shl(b as u32), t1 | t2)
-            },
+            }
             (Value::Symbolic(a, t1), Value::Concrete(b, t2)) => {
                 let bv = make_bv(&a, b, log2(a.get_width()));
                 Value::Symbolic(a.sll(&bv), t1 | t2)
-            },
+            }
             (Value::Concrete(a, t1), Value::Symbolic(b, t2)) => {
                 let bv = make_bv(&b, a, 64);
                 Value::Symbolic(bv.sll(&b.slice(5, 0)), t1 | t2)
-            },
+            }
             (Value::Symbolic(a, t1), Value::Symbolic(b, t2)) => {
-                Value::Symbolic(a.sll(&b.slice(log2(a.get_width())-1, 0)), t1 | t2)
+                Value::Symbolic(a.sll(&b.slice(log2(a.get_width()) - 1, 0)), t1 | t2)
             }
         }
     }
@@ -250,81 +249,84 @@ impl ops::Shl<Value> for Value {
 impl ops::Shr<Value> for Value {
     type Output = Value;
 
-    #[inline] 
+    #[inline]
     fn shr(self, rhs: Value) -> Value {
         match (self, rhs) {
             (Value::Concrete(a, t1), Value::Concrete(b, t2)) => {
                 Value::Concrete(a.wrapping_shr(b as u32), t1 | t2)
-            },
+            }
             (Value::Symbolic(a, t1), Value::Concrete(b, t2)) => {
                 let bv = make_bv(&a, b, log2(a.get_width()));
                 //println!("{:?} {:?}", a, b);
                 Value::Symbolic(a.srl(&bv), t1 | t2)
-            },
+            }
             (Value::Concrete(a, t1), Value::Symbolic(b, t2)) => {
                 let bv = make_bv(&b, a, 64);
                 Value::Symbolic(bv.srl(&b.slice(5, 0)), t1 | t2)
-            },
+            }
             (Value::Symbolic(a, t1), Value::Symbolic(b, t2)) => {
-                Value::Symbolic(a.srl(&b.slice(log2(a.get_width())-1, 0)), t1 | t2)
+                Value::Symbolic(a.srl(&b.slice(log2(a.get_width()) - 1, 0)), t1 | t2)
             }
         }
     }
 }
 
 impl Value {
-
-    #[inline] 
+    #[inline]
     pub fn sdiv(self, rhs: Value) -> Value {
         match (self, rhs) {
             (Value::Concrete(a, t1), Value::Concrete(b, t2)) => {
                 Value::Concrete(((a as i64).wrapping_div(b as i64)) as u64, t1 | t2)
-            },
+            }
             (Value::Symbolic(a, t1), Value::Concrete(b, t2)) => {
                 let bv = make_bv(&a, b, a.get_width());
                 Value::Symbolic(a.sdiv(&bv), t1 | t2)
-            },
+            }
             (Value::Concrete(a, t1), Value::Symbolic(b, t2)) => {
                 let bv = make_bv(&b, a, b.get_width());
                 Value::Symbolic(bv.sdiv(&b), t1 | t2)
-            },
+            }
             (Value::Symbolic(a, t1), Value::Symbolic(b, t2)) => {
                 let width_diff = a.get_width() as i32 - b.get_width() as i32;
                 match width_diff.cmp(&0) {
                     Ordering::Equal => Value::Symbolic(a.sdiv(&b), t1 | t2),
-                    Ordering::Greater => Value::Symbolic(a.sdiv(&b.sext(width_diff as u32)), t1 | t2),
-                    Ordering::Less => Value::Symbolic(a.sext(-width_diff as u32).sdiv(&b), t1 | t2)
+                    Ordering::Greater => {
+                        Value::Symbolic(a.sdiv(&b.sext(width_diff as u32)), t1 | t2)
+                    }
+                    Ordering::Less => Value::Symbolic(a.sext(-width_diff as u32).sdiv(&b), t1 | t2),
                 }
             }
         }
     }
 
-    #[inline] 
+    #[inline]
     pub fn srem(self, rhs: Value) -> Value {
         match (self, rhs) {
             (Value::Concrete(a, t1), Value::Concrete(b, t2)) => {
                 Value::Concrete(((a as i64).wrapping_rem(b as i64)) as u64, t1 | t2)
-            },
+            }
             (Value::Symbolic(a, t1), Value::Concrete(b, t2)) => {
                 let bv = make_bv(&a, b, a.get_width());
                 Value::Symbolic(a.srem(&bv), t1 | t2)
-            },
+            }
             (Value::Concrete(a, t1), Value::Symbolic(b, t2)) => {
                 let bv = make_bv(&b, a, b.get_width());
                 Value::Symbolic(bv.srem(&b), t1 | t2)
-            },
+            }
             (Value::Symbolic(a, t1), Value::Symbolic(b, t2)) => {
                 let width_diff = a.get_width() as i32 - b.get_width() as i32;
                 match width_diff.cmp(&0) {
                     Ordering::Equal => Value::Symbolic(a.srem(&b), t1 | t2),
-                    Ordering::Greater => Value::Symbolic(a.srem(&b.sext(width_diff as u32)), t1 | t2),
-                    Ordering::Less => Value::Symbolic(a.sext(-width_diff as u32).srem(&b), t1 | t2)
+                    Ordering::Greater => {
+                        Value::Symbolic(a.srem(&b.sext(width_diff as u32)), t1 | t2)
+                    }
+                    Ordering::Less => Value::Symbolic(a.sext(-width_diff as u32).srem(&b), t1 | t2),
                 }
             }
         }
     }
 
-    #[inline] 
+    #[inline]
     pub fn asr(self, rhs: Value, sz: u32) -> Value {
         //println!("{:?}, {:?}, {:?}", self, rhs, sz);
         match (self, rhs) {
@@ -332,217 +334,229 @@ impl Value {
                 let shift = 64 - sz as i64;
                 let sign_ext = ((a as i64) << shift) >> shift;
                 Value::Concrete(((sign_ext as i64) >> (b as i64)) as u64, t1 | t2)
-            },
+            }
             (Value::Symbolic(a, t1), Value::Concrete(b, t2)) => {
                 let bv = make_bv(&a, b, log2(sz));
-                Value::Symbolic(a.slice(sz-1, 0).sra(&bv), t1 | t2)
-            },
+                Value::Symbolic(a.slice(sz - 1, 0).sra(&bv), t1 | t2)
+            }
             (Value::Concrete(a, t1), Value::Symbolic(b, t2)) => {
                 let bv = make_bv(&b, a, sz);
-                Value::Symbolic(bv.sra(&b.slice(log2(sz)-1, 0)), t1 | t2)
-            },
+                Value::Symbolic(bv.sra(&b.slice(log2(sz) - 1, 0)), t1 | t2)
+            }
             (Value::Symbolic(a, t1), Value::Symbolic(b, t2)) => {
-                Value::Symbolic(a.slice(sz-1, 0).sra(&b.slice(log2(sz)-1, 0)), t1 | t2)
+                Value::Symbolic(a.slice(sz - 1, 0).sra(&b.slice(log2(sz) - 1, 0)), t1 | t2)
             }
         }
     }
 
-    #[inline] 
+    #[inline]
     pub fn ror(self, rhs: Value, sz: u32) -> Value {
         match (self, rhs) {
             (Value::Concrete(a, t1), Value::Concrete(b, t2)) => {
                 // uhhh
-                let rot = (a & ((1 << sz)-1)).rotate_right(b as u32);
-                let mask = ((1 << b)-1) << (64 - b);
+                let rot = (a & ((1 << sz) - 1)).rotate_right(b as u32);
+                let mask = ((1 << b) - 1) << (64 - b);
                 let val = rot - (rot & mask) + ((rot & mask) >> (64 - sz as u64));
                 Value::Concrete(val, t1 | t2)
-            },
+            }
             (Value::Symbolic(a, t1), Value::Concrete(b, t2)) => {
                 let bv = make_bv(&a, b, log2(sz));
-                Value::Symbolic(a.slice(sz-1, 0).ror(&bv), t1 | t2)
-            },
+                Value::Symbolic(a.slice(sz - 1, 0).ror(&bv), t1 | t2)
+            }
             (Value::Concrete(a, t1), Value::Symbolic(b, t2)) => {
                 let bv = make_bv(&b, a, sz);
-                Value::Symbolic(bv.slice(sz-1, 0).ror(&b.slice(log2(sz)-1, 0)), t1 | t2)
-            },
+                Value::Symbolic(bv.slice(sz - 1, 0).ror(&b.slice(log2(sz) - 1, 0)), t1 | t2)
+            }
             (Value::Symbolic(a, t1), Value::Symbolic(b, t2)) => {
-                Value::Symbolic(a.slice(sz-1, 0).ror(&b.slice(log2(sz)-1, 0)), t1 | t2)
+                Value::Symbolic(a.slice(sz - 1, 0).ror(&b.slice(log2(sz) - 1, 0)), t1 | t2)
             }
         }
     }
 
-    #[inline] 
+    #[inline]
     pub fn rol(self, rhs: Value, sz: u32) -> Value {
         match (self, rhs) {
             (Value::Concrete(a, t1), Value::Concrete(b, t2)) => {
                 // uhhh
                 let na = a << (64 - sz);
                 let rot = na.rotate_left(b as u32);
-                let mask = (1 << b)-1;
+                let mask = (1 << b) - 1;
                 let val = ((rot - (rot & mask)) >> (64 - sz as u64)) + (rot & mask);
                 Value::Concrete(val, t1 | t2)
-            },
+            }
             (Value::Symbolic(a, t1), Value::Concrete(b, t2)) => {
                 let bv = make_bv(&a, b, log2(sz));
-                Value::Symbolic(a.slice(sz-1, 0).rol(&bv), t1 | t2)
-            },
+                Value::Symbolic(a.slice(sz - 1, 0).rol(&bv), t1 | t2)
+            }
             (Value::Concrete(a, t1), Value::Symbolic(b, t2)) => {
                 let bv = make_bv(&b, a, sz);
-                Value::Symbolic(bv.slice(sz-1, 0).rol(&b.slice(log2(sz)-1, 0)), t1 | t2)
-            },
+                Value::Symbolic(bv.slice(sz - 1, 0).rol(&b.slice(log2(sz) - 1, 0)), t1 | t2)
+            }
             (Value::Symbolic(a, t1), Value::Symbolic(b, t2)) => {
-                Value::Symbolic(a.slice(sz-1, 0).rol(&b.slice(log2(sz)-1, 0)), t1 | t2)
+                Value::Symbolic(a.slice(sz - 1, 0).rol(&b.slice(log2(sz) - 1, 0)), t1 | t2)
             }
         }
     }
 
     // get whether values are equivalent
-    #[inline] 
+    #[inline]
     pub fn eq(&self, rhs: &Value) -> Value {
         match (self, rhs) {
             (Value::Concrete(a, t1), Value::Concrete(b, t2)) => {
                 Value::Concrete((*a == *b) as u64, *t1 | *t2)
-            },
+            }
             (Value::Symbolic(a, t1), Value::Concrete(b, t2)) => {
                 let bv = make_bv(&a, *b, a.get_width());
                 Value::Symbolic(a._eq(&bv), *t1 | *t2)
-            },
+            }
             (Value::Concrete(a, t1), Value::Symbolic(b, t2)) => {
                 let bv = make_bv(&b, *a, b.get_width());
                 Value::Symbolic(bv._eq(&b), *t1 | *t2)
-            },
+            }
             (Value::Symbolic(a, t1), Value::Symbolic(b, t2)) => {
                 let width_diff = a.get_width() as i32 - b.get_width() as i32;
                 match width_diff.cmp(&0) {
                     Ordering::Equal => Value::Symbolic(a._eq(&b), *t1 | *t2),
-                    Ordering::Greater => Value::Symbolic(a._eq(&b.uext(width_diff as u32)), *t1 | *t2),
-                    Ordering::Less => Value::Symbolic(b._eq(&a.uext(-width_diff as u32)), *t1 | *t2)
+                    Ordering::Greater => {
+                        Value::Symbolic(a._eq(&b.uext(width_diff as u32)), *t1 | *t2)
+                    }
+                    Ordering::Less => {
+                        Value::Symbolic(b._eq(&a.uext(-width_diff as u32)), *t1 | *t2)
+                    }
                 }
             }
         }
     }
 
     // check if values are *identical*
-    #[inline] 
+    #[inline]
     pub fn id(&self, rhs: &Value) -> Value {
         match (self, rhs) {
             (Value::Concrete(a, t1), Value::Concrete(b, t2)) => {
                 Value::Concrete((a == b) as u64, *t1 | *t2)
-            },
+            }
             (Value::Symbolic(a, t1), Value::Concrete(b, t2)) => {
                 if a.is_const() {
                     Value::Concrete((a.as_u64().unwrap() == *b) as u64, *t1 | *t2)
                 } else {
                     Value::Concrete(0, *t1 | *t2)
                 }
-            },
+            }
             (Value::Concrete(a, t1), Value::Symbolic(b, t2)) => {
                 if b.is_const() {
                     Value::Concrete((b.as_u64().unwrap() == *a) as u64, *t1 | *t2)
                 } else {
                     Value::Concrete(0, *t1 | *t2)
                 }
-            },
+            }
             (Value::Symbolic(a, t1), Value::Symbolic(b, t2)) => {
                 Value::Concrete((a == b) as u64, *t1 | *t2)
             }
         }
     }
 
-    #[inline] 
+    #[inline]
     pub fn slt(&self, rhs: &Value) -> Value {
         match (self, rhs) {
             (Value::Concrete(a, t1), Value::Concrete(b, t2)) => {
                 Value::Concrete(((*a as i64) < (*b as i64)) as u64, *t1 | *t2)
-            },
+            }
             (Value::Symbolic(a, t1), Value::Concrete(b, t2)) => {
                 let bv = make_bv(&a, *b, a.get_width());
                 Value::Symbolic(a.slt(&bv), *t1 | *t2)
-            },
+            }
             (Value::Concrete(a, t1), Value::Symbolic(b, t2)) => {
                 let bv = make_bv(&b, *a, b.get_width());
                 Value::Symbolic(bv.slt(&b), *t1 | *t2)
-            },
+            }
             (Value::Symbolic(a, t1), Value::Symbolic(b, t2)) => {
                 let width_diff = a.get_width() as i32 - b.get_width() as i32;
                 match width_diff.cmp(&0) {
                     Ordering::Equal => Value::Symbolic(a.slt(&b), *t1 | *t2),
-                    Ordering::Greater => Value::Symbolic(a.slt(&b.sext(width_diff as u32)), *t1 | *t2),
-                    Ordering::Less => Value::Symbolic(a.uext((-width_diff) as u32).slt(&b), *t1 | *t2)
+                    Ordering::Greater => {
+                        Value::Symbolic(a.slt(&b.sext(width_diff as u32)), *t1 | *t2)
+                    }
+                    Ordering::Less => {
+                        Value::Symbolic(a.uext((-width_diff) as u32).slt(&b), *t1 | *t2)
+                    }
                 }
             }
         }
     }
 
-    #[inline] 
+    #[inline]
     pub fn slte(&self, rhs: &Value) -> Value {
         self.slt(rhs) | self.eq(rhs)
     }
 
-    #[inline] 
+    #[inline]
     pub fn sgt(&self, rhs: &Value) -> Value {
         !self.slt(rhs) & !self.eq(rhs)
     }
 
-    #[inline] 
+    #[inline]
     pub fn sgte(self, rhs: &Value) -> Value {
         !self.slt(rhs)
     }
-    
-    #[inline] 
+
+    #[inline]
     pub fn ult(&self, rhs: &Value) -> Value {
         match (self, rhs) {
             (Value::Concrete(a, t1), Value::Concrete(b, t2)) => {
                 Value::Concrete((*a < *b) as u64, *t1 | *t2)
-            },
+            }
             (Value::Symbolic(a, t1), Value::Concrete(b, t2)) => {
                 let bv = make_bv(&a, *b, a.get_width());
                 Value::Symbolic(a.ult(&bv), *t1 | *t2)
-            },
+            }
             (Value::Concrete(a, t1), Value::Symbolic(b, t2)) => {
                 let bv = make_bv(&b, *a, b.get_width());
                 Value::Symbolic(bv.ult(&b), *t1 | *t2)
-            },
+            }
             (Value::Symbolic(a, t1), Value::Symbolic(b, t2)) => {
                 let width_diff = a.get_width() as i32 - b.get_width() as i32;
                 match width_diff.cmp(&0) {
                     Ordering::Equal => Value::Symbolic(a.ult(&b), *t1 | *t2),
-                    Ordering::Greater => Value::Symbolic(a.ult(&b.uext(width_diff as u32)), *t1 | *t2),
-                    Ordering::Less => Value::Symbolic(a.uext((-width_diff) as u32).ult(&b), *t1 | *t2)
+                    Ordering::Greater => {
+                        Value::Symbolic(a.ult(&b.uext(width_diff as u32)), *t1 | *t2)
+                    }
+                    Ordering::Less => {
+                        Value::Symbolic(a.uext((-width_diff) as u32).ult(&b), *t1 | *t2)
+                    }
                 }
             }
         }
     }
 
-    #[inline] 
+    #[inline]
     pub fn ulte(&self, rhs: &Value) -> Value {
         self.ult(rhs) | self.eq(rhs)
     }
 
-    #[inline] 
+    #[inline]
     pub fn ugt(&self, rhs: &Value) -> Value {
         !self.ult(rhs) & !self.eq(rhs)
     }
 
-    #[inline] 
+    #[inline]
     pub fn ugte(&self, rhs: &Value) -> Value {
         !self.ult(rhs)
     }
 
-    #[inline] 
+    #[inline]
     pub fn uext(&self, rhs: &Value) -> Value {
         match (self, rhs) {
             (Value::Concrete(a, t1), Value::Concrete(b, t2)) => {
                 Value::Concrete(a & ((1 << b) - 1), *t1 | *t2)
-            },
+            }
             (Value::Symbolic(a, t1), Value::Concrete(b, t2)) => {
                 //let bv = make_bv(&a, b, a.get_width());
                 Value::Symbolic(a.slice(*b as u32 - 1, 0).uext(64 - *b as u32), *t1 | *t2)
-            },
+            }
             (Value::Concrete(a, t), Value::Symbolic(_b, _t)) => {
                 // uh hopefully this doesnt happen
                 Value::Concrete(*a, *t)
-            },
+            }
             (Value::Symbolic(a, t), Value::Symbolic(_b, _t)) => {
                 // uh hopefully this doesnt happen
                 let szdiff = 64 - a.get_width();
@@ -551,21 +565,21 @@ impl Value {
         }
     }
 
-    #[inline] 
+    #[inline]
     pub fn sext(&self, rhs: &Value) -> Value {
         match (self, rhs) {
             (Value::Concrete(a, t1), Value::Concrete(b, t2)) => {
                 let szdiff = 64 - *b as i64;
                 Value::Concrete((((*a as i64) << szdiff) >> szdiff) as u64, *t1 | *t2)
-            },
+            }
             (Value::Symbolic(a, t1), Value::Concrete(b, t2)) => {
                 //let bv = make_bv(&a, b, a.get_width());
                 Value::Symbolic(a.slice(*b as u32 - 1, 0).sext(64 - *b as u32), *t1 | *t2)
-            },
+            }
             (Value::Concrete(a, t), Value::Symbolic(_b, _t)) => {
                 // uh hopefully this doesnt happen
                 Value::Concrete(*a, *t)
-            },
+            }
             (Value::Symbolic(a, t), Value::Symbolic(_b, _t)) => {
                 // uh hopefully this doesnt happen
                 let szdiff = 64 - a.get_width();
@@ -574,38 +588,41 @@ impl Value {
         }
     }
 
+    #[inline]
     pub fn slice(&self, high: u64, low: u64) -> Value {
         match self {
             Value::Concrete(a, t) => {
-                Value::Concrete(*a & (((1 << (high-low+1))-1) << low), *t)
-            },
-            Value::Symbolic(a, t) => {
-                Value::Symbolic(a.slice(high as u32, low as u32), *t)
+                Value::Concrete(*a & (((1 << (high - low + 1)) - 1) << low), *t)
             }
+            Value::Symbolic(a, t) => Value::Symbolic(a.slice(high as u32, low as u32), *t),
         }
     }
 
+    #[inline]
     pub fn as_u64(&self) -> Option<u64> {
         match self {
             Value::Concrete(a, _t) => Some(*a),
-            Value::Symbolic(a, _t) => a.as_u64()
+            Value::Symbolic(a, _t) => a.as_u64(),
         }
     }
 
+    #[inline]
     pub fn as_bv(&self) -> Option<BitVec> {
         match self {
             Value::Concrete(_a, _t) => None,
-            Value::Symbolic(a, _t)  => Some(a.to_owned())
+            Value::Symbolic(a, _t) => Some(a.to_owned()),
         }
     }
 
+    #[inline]
     pub fn get_taint(&self) -> u64 {
         match self {
             Value::Concrete(_a, t) => *t,
-            Value::Symbolic(_a, t) => *t
+            Value::Symbolic(_a, t) => *t,
         }
     }
 
+    #[inline]
     pub fn with_taint(&self, taint: u64) -> Value {
         match self {
             Value::Concrete(a, t) => Value::Concrete(*a, *t | taint),
@@ -616,84 +633,74 @@ impl Value {
     pub fn depends(&self, rhs: &Value) -> bool {
         match rhs {
             Value::Concrete(_a, t1) => self.get_taint() & *t1 != 0,
-            Value::Symbolic(abv, t1) => {
-                match self {
-                    Value::Concrete(_b, t2) => *t1 & *t2 != 0,
-                    Value::Symbolic(bbv, t2) => {
-                        let mut tainted = false;
-                        if let Some(sym) = abv.get_symbol() {
-                            tainted = format!("{:?}", bbv).contains(sym);
-                        }
-                        tainted || (*t1 & *t2 != 0)
+            Value::Symbolic(abv, t1) => match self {
+                Value::Concrete(_b, t2) => *t1 & *t2 != 0,
+                Value::Symbolic(bbv, t2) => {
+                    let mut tainted = false;
+                    if let Some(sym) = abv.get_symbol() {
+                        tainted = format!("{:?}", bbv).contains(sym);
                     }
+                    tainted || (*t1 & *t2 != 0)
                 }
-            }
+            },
         }
     }
 
-    #[inline] 
+    #[inline]
     pub fn is_concrete(&self) -> bool {
-        if let Value::Concrete(_, _) = self {
-            true
-        } else {
-            false
-        }
+        matches!(self, Value::Concrete(_, _))
     }
 
-    #[inline] 
+    #[inline]
     pub fn is_symbolic(&self) -> bool {
-        if let Value::Symbolic(_, _) = self {
-            true
-        } else {
-            false
-        }
+        matches!(self, Value::Symbolic(_, _))
     }
 
-    #[inline] 
+    #[inline]
     pub fn add(&self, rhs: &Value) -> Value {
         wrapping_binary_ops!(self, rhs, add, wrapping_add)
     }
 
-    #[inline] 
+    #[inline]
     pub fn sub(&self, rhs: &Value) -> Value {
         wrapping_binary_ops!(self, rhs, sub, wrapping_sub)
     }
 
-    #[inline] 
+    #[inline]
     pub fn mul(&self, rhs: &Value) -> Value {
         wrapping_binary_ops!(self, rhs, mul, wrapping_mul)
     }
 
-    #[inline] 
+    #[inline]
     pub fn div(&self, rhs: &Value) -> Value {
         wrapping_binary_ops!(self, rhs, udiv, wrapping_div)
     }
 
-    #[inline] 
+    #[inline]
     pub fn rem(&self, rhs: &Value) -> Value {
         wrapping_binary_ops!(self, rhs, urem, wrapping_rem)
     }
 
-    #[inline] 
+    #[inline]
     pub fn and(&self, rhs: &Value) -> Value {
         binary_ops!(self, rhs, and, &)
     }
 
-    #[inline] 
+    #[inline]
     pub fn or(&self, rhs: &Value) -> Value {
         binary_ops!(self, rhs, or, |)
     }
 
-    #[inline] 
+    #[inline]
     pub fn xor(&self, rhs: &Value) -> Value {
         binary_ops!(self, rhs, xor, ^)
     }
-    
-    #[inline] 
+
+    #[inline]
     pub fn size(&self) -> u32 {
         match self {
             Value::Concrete(_a, _t) => 64,
-            Value::Symbolic(a, _t) => a.get_width()
+            Value::Symbolic(a, _t) => a.get_width(),
         }
     }
-}
\ No newline at end of file
+}
